package com.tcnd.wasdg.dataaccess;

// #SIGNATURE#WASBOOK#

import java.util.*;
import java.sql.*;
import java.io.*;

import com.tcnd.wasdg.common.*;

public class DataAccessComponent
{
	private java.sql.Connection conn;
	private static final boolean SUCCESS = true;
	private static final boolean FAILURE = false;
	private boolean someData;
	private static final String dbguidePropertiesFileName="/BOOK/10/Code/dbguide.properties";
	private String dbDriver;
	private String dbUrl;
	private String dbUser;
	private String dbPassword;


	public DataAccessComponent()
	{
		super();
		loadDefaultConnectionSettings();
	}

	private void loadDefaultConnectionSettings()
	{
		java.util.Properties prop;
		prop = new Properties();
		try
		{
			prop.load(new FileInputStream(dbguidePropertiesFileName));
		} catch (Exception e)
		{
			e.printStackTrace();
		}

		dbDriver = prop.getProperty("dbdriver");
		dbUrl = prop.getProperty("dburl");
		dbUser = prop.getProperty("dbuser");
		dbPassword = prop.getProperty("dbpassword");
	}


	private void initDefaultConnectionSettings (IDataAccessObject iDAO)
	{


		iDAO.setDBdriver(dbDriver);

		iDAO.setDBurl(dbUrl);

		iDAO.setDBuser(dbUser);

		iDAO.setDBpassword(dbPassword);

		String dbu = iDAO.getDBuser();
		String dbpasswd = iDAO.getDBpassword();

		// If user and password are empty, set to null so that the implicit database login will be used
		if (dbUser != null && (dbUser.equals("null") || dbUser.equals(""))) iDAO.setDBuser(null);
		if (dbPassword != null && (dbPassword.equals("null") || dbPassword.equals(""))) iDAO.setDBpassword(null);

		// An exception should be thrown as explained in Chapter 23
		// if((dbUrl==null || dbUrl.equals("")) || (dbUser==null || dbUser.equals("")) || (dbPassword==null || dbPassword.equals(""))) {
		// e = new WasdgException();
		// ...
	}


	// private void connect(IDataAccessObject iDAO) throws WASDGException {
	private void connect(IDataAccessObject iDAO)
	{
		//con = DriverManager.getConnection("jdbc:weblogic:jts:" +getJdbcProperty("poolName"));

		if ( (iDAO.getDBuser() == null) || (iDAO.getDBpassword() == null) || (iDAO.getDBurl() == null) || (iDAO.getDBdriver() == null))
		{
			initDefaultConnectionSettings(iDAO);	
		}
		try
		{
			String dbu = iDAO.getDBuser();
			String dbpasswd = iDAO.getDBpassword();
			String url = iDAO.getDBurl();
			String dbdriver = iDAO.getDBdriver();

			// The Class.forName is not set explicitly but read from the properties
			// Class.forName("COM.ibm.db2.jdbc.net.DB2Driver");
			Class.forName(dbdriver);
			conn = DriverManager.getConnection(url,dbu,dbpasswd);
		} catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	// private void disconnect() throws WASDGException {
	private void disconnect()
	{
		try
		{
			if (conn != null)
				conn.close();
		} catch (SQLException sqe)
		{
			// wrap in WASDG message here (Chapter 23)
			// throw new WasdgException()
		}
	}


	public IDataAccessObject executeQuery(IDataAccessObject iDAO)
	{

		// The command is assumed failing unless it passes the whole executeQuery routine
		iDAO.setCommandResult(FAILURE);
		// No data has been retrieved in the dataSet
		someData = false;

		String name=null;

		if (iDAO == null)
		{
			return null;
		}

		try
		{
			connect(iDAO);

			if (iDAO.getCommandString() == null)
			{
				return iDAO;
			}

			// Substitute the marked string in the SQL statement with the appropriate parameters
			name = stmtParametersBinding(iDAO);

			Statement stmt = conn.createStatement();

			// set the Max Records, if any exists in DataAccess Interface Object
			if (iDAO.getMaxRows() > 0)
			{
				stmt.setMaxRows(iDAO.getMaxRows());
			}

			ResultSet rs = stmt.executeQuery(name);

			// store results in the data-set in the iDAO
			DataSet ds = storeResults(rs);
			iDAO.setDataSet(ds);

			if (stmt != null)
			{
				stmt.close();
				// Now the command is successful, check also if we've got data
				if (someData)
					iDAO.setCommandResult(SUCCESS);
			}
		} catch (SQLException sqe)
		{
			// wrap in WASDG message here (Chapter 23)
			//throw new WasdgException()
		}

		disconnect();
		return iDAO;
	}


	public IDataAccessObject executeUpdate(IDataAccessObject iDAO)
	{

		// The command is assumed failed unless it passes the whole executeUpdate routine
		iDAO.setCommandResult(FAILURE);
		String name=null;

		if (iDAO == null)
		{
			return null;
		}

		try
		{
			connect(iDAO);

			if (iDAO.getCommandString() == null)
			{
				return iDAO;
			}

			// Substitute the marked string in the SQL statement with the appropriate parameters
			name = stmtParametersBinding(iDAO);

			Statement stmt = conn.createStatement();

			// set the Max Records, if any exists in DataAccess Interface Object
			if (iDAO.getMaxRows() > 0)
			{
				stmt.setMaxRows(iDAO.getMaxRows());
			}

			// ResultSet rs = stmt.executeQuery(name);
			stmt.executeUpdate(name);
			int updateCount = stmt.getUpdateCount();
			if (stmt != null)
			{
				stmt.close();
				// Now the command is successful
				if (updateCount > 0) iDAO.setCommandResult(SUCCESS);
			}
		} catch (SQLException sqe)
		{
			// wrap in WASDG message here (Chapter 23)
			// 	throw new WasdgException()
			System.out.println(sqe);
		}
		disconnect();
		return iDAO;
	}

	
	// Replace marked tokens in a statement command string with their correponding values.
	// The tokens and their values are stored in a hash table, and the colon is used as a marker.
	// The parameter iDAO contains the original command string and name/value pairs
	// Return the command string after all substitution are made
	private String stmtParametersBinding(IDataAccessObject iDAO)  
	{
		String Command=iDAO.getCommandString();

		if (iDAO.getParameters() == null || iDAO.getParameters().isEmpty())
			return Command;

		for (Enumeration e = iDAO.getParameters().keys() ; e.hasMoreElements() ;)
		{
			Object param = e.nextElement();	  

			String paramName  = param.toString();
			String paramValue = iDAO.getParameters().get(param).toString();

			if (paramName != null && paramValue != null)
			{
				Command = StrTool.replaceUnsensitive( Command, ":" + paramName, paramValue );				  
			}
		}	   
		return Command;
	}


	private DataSet storeResults(ResultSet rs) throws java.sql.SQLException {
		DataSet dataSet = new DataSet();
		try
		{
			//metadata gives you column names and number of columns associated
			//with the result set
			ResultSetMetaData rsmd = rs.getMetaData();
			int columnCount = rsmd.getColumnCount();
			for (int x=1;x<=columnCount;x++)
			{
				String columnName = rsmd.getColumnName(x);
				dataSet.setColName(columnName,x - 1);	   
			}

			//all values are stored as strings
			while (rs.next())
			{
				someData = true;
				for (int x=1;x<=columnCount;x++)
				{
					dataSet.addToRow(rs.getString(x));
				}
				dataSet.addRow();
			}

		} catch (SQLException sqe)
		{
			int sqeErrCode = sqe.getErrorCode();
			if (sqeErrCode < 50000)
				System.out.println("SQL Exception " + sqe.getMessage());
		} catch (Exception e)
		{
			try
			{
				System.out.println(e.toString());
			} catch (Exception ignore)
			{
			}
		}
		return dataSet;
	}

	// Creates an instance of the IDataAccesseObject and returns a reference to the instance.
	// This reference can then be used to update with settings to connect to a database and execute a sql statement
	// or stored procedure.
	// Return com.tcnd.wasdg.common.IDataAccessObject
	public IDataAccessObject createDAObject()
	{
		IDataAccessObject iDAO = new IDataAccessObject();
		return iDAO;
	}

	public IDataAccessObject fetchEmpAccount(IDataAccessObject iDAO)
	{
		// The command is assumed failed unless it passes the whole routine
		iDAO.setCommandResult(FAILURE);
		// No data fetched
		someData = false;

		String name = null;

		if (iDAO == null)
		{
			return null;
		}

		try
		{
			connect(iDAO);

			if (iDAO.getCommandString() == null)
			{
				return iDAO;
			}

			// bind the sql statement with parameters
			// Substitute the marked string in the SQL statement with the appropriate parameters
			name = stmtParametersBinding(iDAO);

			Statement stmt = conn.createStatement();

			// set the Max Records, if any exists in DataAccess Interface Object. Only 1 in here!
			if (iDAO.getMaxRows() > 0)
			{
				stmt.setMaxRows(iDAO.getMaxRows());
			}

			ResultSet rs = stmt.executeQuery(name);

			EmpAccnt aEmp = new EmpAccnt();
			if (rs.next())
			{
				aEmp.setUserID(rs.getString(1));
				aEmp.setFrstName(rs.getString(2));
				aEmp.setLastName(rs.getString(3));
				aEmp.setImage(rs.getString(4));
				aEmp.setSsn(rs.getString(5));
				aEmp.setEmpno(rs.getString(6));
				aEmp.setEmail(rs.getString(7));
				aEmp.setContact(rs.getString(8));
				someData = true;
			}

			iDAO.setEmpAccnt(aEmp);

			if (stmt != null)
				stmt.close();
				if (someData)
					iDAO.setCommandResult(SUCCESS);
		} catch (SQLException sqe)
		{
			int sqeErrCode = sqe.getErrorCode();
			if (sqeErrCode < 50000)
				System.out.println("SQLException message" + sqe.getMessage());
		}
		disconnect();
		return iDAO;
	}

	public IDataAccessObject fetchCliAccount(IDataAccessObject iDAO)
	{
		// The command is assumed failed unless it passes the whole routine
		iDAO.setCommandResult(FAILURE);

		// No data fetched yet
		someData = false;

		String name = null;

		if (iDAO == null)
		{
			return null;
		}
		try
		{
			connect(iDAO);

			if (iDAO.getCommandString() == null)
			{
				return iDAO;
			}

			// Substitute the marked string in the SQL statement with the appropriate parameters
			name = stmtParametersBinding(iDAO);

			Statement stmt = conn.createStatement();

			// set the Max Records, if any exists in DataAccess Interface Object. Only 1 in here!
			if (iDAO.getMaxRows() > 0)
			{
				stmt.setMaxRows(iDAO.getMaxRows());
			}

			ResultSet rs = stmt.executeQuery(name);

			CliAccnt aCli = new CliAccnt();
			if (rs.next())
			{
				aCli.setClientID(rs.getString(1));
				aCli.setBankID(rs.getString(2));
				aCli.setCategory(rs.getString(3));
				aCli.setRempID(rs.getString(4));
				aCli.setFrstName(rs.getString(5));
				aCli.setLastName(rs.getString(6));
				aCli.setImage(rs.getString(7));
				aCli.setEmail(rs.getString(8));
				aCli.setCdebits(rs.getInt(9));
				aCli.setCcredits(rs.getInt(10));
				aCli.setBalance(rs.getInt(11));
				someData = true;
			}

			iDAO.setCliAccnt(aCli);
			CliAccnt zz = iDAO.getCliAccnt();	

			if (stmt != null)
				stmt.close();
				if (someData)
					iDAO.setCommandResult(SUCCESS);
		} catch (SQLException sqe)
		{
			int sqeErrCode = sqe.getErrorCode();
			if (sqeErrCode < 50000)
				System.out.println("SQLException message" + sqe.getMessage());
		}
		disconnect();
		return iDAO;
	}
}
