package com.tcnd.wasdg.dbreg;

import java.util.*;
import java.io.*;
import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;

public class DbregLoginModule implements LoginModule
{

    private Subject subject;
    private CallbackHandler callbackHandler;
    private DbregPrincipal principal;
    private Map cacheMap;
    private Map options;
    private boolean debug;

    // State information for the currently-authenticated user
    private String loginname = "dbreg";
    private String dbregloginname = null;
    private char[] password = {'p','w','d','0','0','7'};
    private char[] dbregpassword = new char[80];
    private int length_dbregpassword;

    private boolean succeeded = false;
    private boolean commitSucceeded = false;

    private static final String dbguidePropertiesFileName="/BOOK/20/dbreg_sysprotected/dbreg_login.properties";

    private void loadDefaultConnectionSettings()
    {
    	java.util.Properties prop;
	prop = new Properties();
	try
	    {
		prop.load(new FileInputStream(dbguidePropertiesFileName));
	    } 
	catch (Exception e)
	    {
		e.printStackTrace();
	    }
	dbregloginname = prop.getProperty("dbregloginname");
	String s = prop.getProperty("dbregpassword");
	s.getChars(0,s.length(),dbregpassword,0);
	length_dbregpassword = s.length();
	
	// Print these to make sure we have the correct data
	System.err.println(" -- dbreg_login.properties: " + dbregloginname);
	System.err.println(" -- dbreg_login.properties: " + s);

    }

    public void initialize(Subject s, CallbackHandler cb, Map cacheMap, Map options) 
    {
        subject = s;
        callbackHandler = cb;
	cacheMap = cacheMap; 
	options = options;


	loadDefaultConnectionSettings();
        // initialize any configured options
        debug = "true".equalsIgnoreCase((String)options.get("debug"));

        // We don't use the shared map to cache results between
        // attempts, but otherwise wed need to save it here.
    }

    public boolean login() throws LoginException 
    {
        if (debug)
            System.err.println("DbregLoginModule login() " + dbregloginname);

        // Authenticate user at this point.
        // Possible to instantiate callback objects
        // and put them in an array and call the callback handler.
	// Use cacheMap to cache previous login. Not needed in this case
	// because the registrar will authenticate only once at servlet
	// login.
	// Users who are authenticated are usually in a session that will last
	// several minutes, and new users will overwrite the cache, therefore
	// caching is useless in our situation.

        // Normally, we'd set this from the getName() method of
        // the name callback, or from the user environment.

        // Check password and return true if correct
	if (loginname.equals(dbregloginname) && (password.length == length_dbregpassword))
	    {
		int i = 0;
		while ( i < length_dbregpassword )
		    {
			if ( password[i] != dbregpassword[i] ) break;
			i++;
		    }
		if (i == password.length)
		    succeeded = true;
		else
		    succeeded = false;
	    }
	else
	    succeeded = false;

	if (succeeded)
	    return true;
	else
	    {
		loginname = null;
		for (int i = 0; i < password.length; i++)
		    password[i] = ' ';
		password = null;
		throw new FailedLoginException("Hmm!");
	    }
    }

    public boolean commit() throws LoginException 
    {
        if (debug)
            System.err.println("DbregLoginModule commit()");

        if (!succeeded) 
	    {
		// We didn't authenticate the user, but someone else did.
		// Clean up our state, but don't add our principal to
		// the subject
		loginname = null;
		return false;
	    }

        principal = new DbregPrincipal(loginname);
        // defaultUser might already be in the subject if another module
        // authenticated him.
        if (!subject.getPrincipals().contains(principal)) 
	    {
		subject.getPrincipals().add(principal);
	    }

        // Clean up our internal state
        loginname = null;
        commitSucceeded = true;
        return true;
    }

    public boolean abort() throws LoginException 
    {
        if (debug)
            System.err.println("DbregLoginModule abort()");

        if (succeeded == false)
            // We failed, and so did someone else, so just clean up
    	    return false;
        else if (succeeded == true && commitSucceeded == true) 
	    {
		// Our login succeeded, but another required module failed
		// We must remove our principal and clean up
		logout();
	    } 
	else  
	    {
		// Our commit failed, even though login succeeded
		// The rest of our internal state should already
		// have been cleaned up
		succeeded = false;
	    }
        return true;
    }

    public boolean logout() throws LoginException 
    {
        if (debug)
            System.err.println("DbregLoginModule logout()");

        subject.getPrincipals().remove(principal);
        principal = null;
        loginname = null;
        succeeded = commitSucceeded = false;
        return true;
    }
}
