package com.tcnd.wasdg.common;

// #SIGNATURE#WASBOOK#

import java.io.*;
import java.util.*;

// Once data is accessed using the DataAccessComponent class, the data 
// can be stored in an object of type DataSet.
// The DataSet is a helper class to store results from DAC
// This class is a referenced from within IDataAccessObject with the method 
// getDataSet() and this is the way a program can fetch the result of the query.
// 
// A HashTable is used to store (column name,value) pair. 
// The data set structure is used to contain result set type of data.
//
// Once you get an instance of the IDataAccessObject, you can set your DataSet
// from the ResultSet that is returned by the stmt.executeQuery() of an open connection.
// However, before getting the data from the DataSet, check if the getCommandResult() is 
// true to make sure that there is any data available. 
// Refer to the example of Chapter 10.



public class DataSet implements java.io.Serializable
{
	Hashtable   ht;
	Vector	  rows;
	Vector	  row;
	int		 rowsIndex = 0;
	Vector	  currentRow;
	private static final boolean SUCCESS = true;
	private static final boolean FAILURE = false;

	
	 // The DataSet object stores results. 
	 // The ht Hashtable stores
	 //    the column name as a key and the column index as a value.
	 // The rows vector contains all the row objects.
	 // The row vector contains all the string values of one row
	 //    and is used internally while computing the DataSet object.
	 
	public DataSet()
	{
		ht = new Hashtable();
		row = new Vector(); 
		rows = new Vector();
		currentRow = new Vector();
	}

	// Adds a row to the rows vector. Used by DataAccessComponentBean class
	public void addRow()
	{
		rows.addElement(row);
		row = new Vector();
	}

	// Adds a column value into the row. Used by SPDataAccessBean class
	public void addToRow(String rsCol)
	{
		row.addElement(rsCol);  
	}

	// get the number of columns in a row of the rowset
	public int getColumnCount()
	{
		int rc;

		if (ht.isEmpty())
			rc = 0;
		else
			rc = ht.size();
		return rc;
	}

	// Positions to the first row in the results and return boolean on success
	public boolean seekFirstRow()
	{
		boolean rowsRC; 
		if (rows.isEmpty())
			rowsRC = false;
		else
		{
			rowsIndex = 0;
			rowsRC = true;
		}  
		return rowsRC;  
	}

	// Positions to the next row in the results and return boolean on success
	public boolean seekNextRow()
	{
		boolean rowsRC;

		if ((rowsIndex+ 1) < rows.size())
		{
			rowsIndex++;

			rowsRC = true;
		} else
			rowsRC = false;

		return rowsRC;  
	}

	// Positions the dataCache to the row number requested and return boolean on success
	public boolean seekRow(int row)
	{
		boolean b;
		if (row < rows.size())
		{
			rowsIndex = row;
			b = true;
		} else
			b = false;
		return b;
	}

	// Get the value at the requested row and index (column) in the row. Return 
	// the value for the requested (row,column)
	// Return null if that (row,column) is empty
	public String getRowCol(int row, int index)
	{
		String rc;
		if (row < rows.size())
		{
			rowsIndex = row;
			rc = getValue(index);

		} else
			rc = null;
		return rc;  
	}

	// Get the value at the requested row for a specific column (or field) name
	// Return String the value if that cell is not empty, otherwise null
	public String getRowCol(int row, String name)
	{
		String rc;
		if (row < rows.size())
		{
			rowsIndex = row;
			rc = getValue(name);
		} else
			rc = null;
		return rc;  
	}


	// Get the number of rows in the results if any, otherwise return 0
	public int getRowCount()
	{
		int rc;
		if (rows.isEmpty()) rc = 0;
		else rc = rows.size();

		return rc;
	}


	// Get the string value at a specific index (column) in the current row
	// of the results. If value stored at (current row, column index) then return null.
	public String getValue(int index)
	{
		String v;
		if (rows.isEmpty()) v = null;
		else
		{
			setRow();
			if (index < currentRow.size())
				v = (String)currentRow.elementAt(index);
			else
				v = null;
		}

		return v;
	}

	// Get the string value at a specific column name (or field name) in the current row
	// of the results. If value stored at (current row, column name) then return null.
	public String getValue(String name)
	{
		String v;
		String htValue;
		// convert name to upper to hit on column name.
		name = name.toUpperCase();
		htValue = (String)ht.get(name);
		int index = 0;

		if (htValue == null)
			v = null;
		else
		{
			index = Integer.parseInt(htValue);
			setRow();
			if (index < currentRow.size())
			{
				v = (String)currentRow.elementAt(index);
			} else
				v = null;
		}

		return v;
	}

	// Set the Hashtable structure with a column name for the key, 
	// and the value is set as an index into the row vector for the
	// column name.
	public void setColName(String colName,int colIndex)
	{
		colName = colName.toUpperCase();
		String colIn = Integer.toString(colIndex);
		ht.put(colName,colIn);
	}


	// Sets the current row to rowsIndex
	private void setRow()
	{
		currentRow = (Vector)rows.elementAt(rowsIndex);
	}


	// Set the value at the index of the current row with a String value
	// Return true on success and false on failure
	public boolean setValue(int index, String value)
	{
		boolean b;
		b = FAILURE;
		if (rows.isEmpty()) b = FAILURE;
		else
		{
			setRow();
			if (index < currentRow.size() && index>=0)
			{
				currentRow.removeElementAt(index);
				currentRow.insertElementAt(value,index);
				b = SUCCESS;
			}
		}
		return b;
	}

	// Set the value at the column name of the current row with a String value
	// Return true on success and false on failure
	public boolean setValue(String colName, String colValue)
	{
		String htValue;
		boolean b;
		b = FAILURE;
		// convert column name to upper
		colName = colName.toUpperCase();
		htValue = (String)ht.get(colName);
		int index = 0;

		if (htValue == null) b = FAILURE;
		else
		{
			index = Integer.parseInt(htValue);
			setRow();
			if (index < currentRow.size() && index>=0)
			{
				currentRow.removeElementAt(index);
				currentRow.insertElementAt(colValue,index);
				b = SUCCESS;
			}
		}
		return b;
	}
}

