package com.tcnd.wasdg.common;

//import java.util.PropertyResourceBundle;
//import java.util.Locale;
import java.util.*;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;

public class BundleManager
{
	private static final String wasdgPropertiesFileName="/BOOK/23/Code/Properties/base.properties";
	private static String baseFile = "base";
	private static Hashtable mBundles;

private BundleManager() {super();}

private static PropertyResourceBundle createBundle(String file)
{
	PropertyResourceBundle baseBundle = null;
	try
	{
		baseBundle =  new PropertyResourceBundle(
			new BufferedInputStream(new FileInputStream(file + ".properties")));
	}
	catch (Exception IOException)
	{
		System.out.println("BundleManager.createBundle: File " + file + ".properties not found!");
	}
	return baseBundle;
}
	
public static void init() throws Exception
{
	String baseDir;
	String fileName;
	Vector baseList;
	PropertyResourceBundle baseBundle;

	java.util.Properties prop;
	prop = new Properties();
	try
	{
		prop.load(new FileInputStream(wasdgPropertiesFileName));
	} catch (Exception e)
	{
		e.printStackTrace();
	}
	baseDir = prop.getProperty("properties.dir");
	baseBundle = createBundle(baseDir + "/" + baseFile);

	mBundles = new Hashtable();
	mBundles.put("base",baseBundle);

	baseList = listFiles(baseDir);
	for (int i = 0; i < baseList.size(); i++ )
	{
		fileName = (String) baseList.elementAt(i);
		baseBundle = createBundle(baseDir + "/" + fileName);
		mBundles.put(fileName, baseBundle);
	}
}

public static PropertyResourceBundle getBundle( String bundleName )
{
	try
	{
		if (mBundles == null) BundleManager.init();
		return (PropertyResourceBundle) mBundles.get( bundleName );
	}
	catch (Exception e) 
	{
		return null;
	}
}

// Get a property from a cached bundle
public static String getString(String bundleName, String propertyName) 
{
	try
	{
		if (mBundles == null) BundleManager.init();
		PropertyResourceBundle bundle = getBundle(bundleName);
		if (bundle == null) return ""; 
		return bundle.getString(propertyName);
	}
	catch (Exception e)
	{
		return "";
	}
}

// Given the base directory (baseDir), return a list of all properties 
// files in specified directory, otherwise return an empty Vector.
private static Vector listFiles(String baseDir) 
{
	Vector filesList = new Vector();
	int	k;
	try
	{
		File dir = new File(baseDir);
		String[] files = dir.list();
		
		for (int i=0; i<files.length; i++)
		{
			String fileName = files[i];
			k = fileName.lastIndexOf(".");
			if (k > 0)
				fileName = fileName.substring(0, k);
			filesList.addElement(fileName);
		}
	}
	catch (Exception e)
	{
	}
	return filesList;
}
}
