package com.tcnd.wasdg.log;

import com.tcnd.wasdg.common.BundleManager;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * This class provides logging services for applications tracing.
 * The logging API is:
 * <p><code>
 * Log.log( "message" );
 * <p>
 * Two writers are provided: LogWriterStdout and LogWriterFile.
 * The file "log.properties" contains the location where the log file
 * is created, and whether to write data to stdout or to a file.
 * <p><code>
 * LogTo=FILE			- write to a file
 * LogFile=/tmp/wasdg.log	- logfile name
 * LogBackup=3			- Number of logs to rotate
 * </code>
 */
public class Log 
{
	private static LogWriter	     lWriter;
	// timestamp format
	private static SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yy HH:mm:ss:SSS");

// Default constructor is private. Do not instantiate.
private Log() 
{
	super();
}

// Write messages to output
public static void flush()
{
	lWriter.flushWriter();
}

public static void init() 
{
	// Log to FILE
	if (BundleManager.getString("log","LogTo").compareTo("FILE") == 0)
	{
		String LogName = BundleManager.getString("log","LogFile");
		String LogBackup = BundleManager.getString("log","LogBackup");
		if (LogName.equals("")) LogName = "foo.log";
		if ( LogBackup.equals("") )
			lWriter = new LogWriterFile(LogName);
		else
			lWriter = new LogWriterFile(LogName,Integer.parseInt(LogBackup));
		lWriter.startWriter();
	}
	else // to STDOUT
	{
		lWriter = new LogWriterStdout();
 		lWriter.startWriter();
	}
}

// Write a message to the log.
// The formatted message is shipped out: timestamp \t message
public static void log(String message )  
{
	Calendar    longDate;
	String      datedMessage;

	if (null == lWriter) init();
	
	// Add current date & time to message
	longDate = GregorianCalendar.getInstance(TimeZone.getDefault());
	datedMessage = dateFormat.format(longDate.getTime())+ "\t" + message;
	lWriter.writeMessage(datedMessage);
}

public static void stopLog() 
{
	if (null == lWriter) return;
	lWriter.stopWriter();
}
}
