package com.tcnd.wasdg.log;

import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.File;

// Extends LogWriter to write log messages to a file
public class LogWriterFile extends LogWriter 
{
	private String currentFileName;
	private PrintWriter fStream;
	private int numToBack;

// Constructor with fileName
public LogWriterFile(String fileName) 
{
	super();
	currentFileName = fileName;
	init(); // open a new log
}

// Constructor with a filename and number of backups
public LogWriterFile(String fileName, int logBackup) 
{
	super();
	currentFileName = fileName;
	numToBack  = logBackup;
	init(); // open a new log and backup previous logs
}

// Start a new log.  Rotate old logs by appending a number to an
// old log file. More recent logs have lower number.
public synchronized void init() 
{
	File lLogFile;

	// Erase backup n
	lLogFile = new File(currentFileName + "." + numToBack);
	if (lLogFile.exists()) 
	{ 
		System.out.println("LogWriterFile.init: unlink " + lLogFile.getName());
		lLogFile.delete();
	}

	// Rename (n-1 through 1) to (n through 2)
	for (int i=numToBack-1; i>0; i--)
	{
		lLogFile = new File( currentFileName + "." + i);
		if ( lLogFile.exists() )
		{
			 System.out.println("LogWriterFile.init: rename " + currentFileName + "." + (i) + " to " + currentFileName + "." + (i+1));
			lLogFile.renameTo(new File(currentFileName + "." + (i+1)));
		}
	}

	lLogFile = new File(currentFileName);
	if (lLogFile.exists())
	{
	   	System.out.println("LogWriterFile.init: rename " + currentFileName + " to " + currentFileName + ".1");
	   	lLogFile.renameTo( new File( currentFileName + ".1"));
	}
	startWriter();
}

// Initialize the writer and start it up by opening the file specified in the constructor
public void startWriter() 
{
	try
	{
		fStream = new PrintWriter( 
			new BufferedWriter(new FileWriter(currentFileName)));
		writeMessage("LogWriterFile.startWriter: Start the " + currentFileName);
	}
	catch (Exception e)
	{
		e.printStackTrace();
	}
}

// Write all pending messages to output
public void flushWriter() 
{
	writeMessage("LogWriterFile.flushWriter: flush() the " + currentFileName);
	fStream.flush();
}

// Stop the writer, write all pending messages, and close the file
public void stopWriter() 
{
	writeMessage("LogWriterFile.stopWriter: Stop the " + currentFileName);
	fStream.flush();
	fStream.close();
}

// Write one message to the destination.
public void writeMessage(String aMessage) 
{
	fStream.println(aMessage);
	fStream.flush();
}
}
