package com.tcnd.wasdg.wasdgexceptions;

import com.tcnd.wasdg.log.Log;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.util.*;
import java.lang.*;

// WasdgException class extends Exception to support multiple messages in a single exception.
// In debug mode new exceptions can hold references to old exeptions.
// getStackTraceString() get information from the entire chain

public class WasdgException extends Exception 
{
	// Control nesting for debugging
	public static boolean debugMode = true;
	// Chain of exceptions
	private Throwable nestedChainException = null;
	// Stack trace from nested exceptions
	private String nestedStackTrace = null;
	// Support for multiple messages
	private Vector arrayMessages = new Vector(10);

// Default constructor, no message or nested exception
public WasdgException() 
{
	super();
	init( null, null );
}

// Constructor with mnemonic
public WasdgException(String mnemonic) 
{
	super(mnemonic);
	init(mnemonic,null);
}

// Constructor with nested exception
public WasdgException(Throwable eThrowable)
{
	super();
	init( null, eThrowable );
}

// Constructor with mnemonic and nested exception
public WasdgException(String mnemonic, Throwable eThrowable)
{
	super(mnemonic);
	init( mnemonic, eThrowable );
}

// Converts the stack trace information of the provided exception to a string
static public String generateStackTraceString(Throwable eThrowable) 
{
	StringWriter lWriter = new StringWriter();
	eThrowable.printStackTrace(new PrintWriter(lWriter));
	return lWriter.toString();
}

// Returns nested exception, null otherwise
public Throwable getNestedException() 
{
	return nestedChainException;
}

// Returns the stack trace string information about this exception and all nested ones
public String getStackTraceString()
{
	return nestedStackTrace;
}

// On creation of a new exception, store the mnemonic and stack trace from this exception
// In debug mode: log the new exception, get stacktrace information from any nested 
// exception and chain to the nested exception.
 
private void init(String mnemonic, Throwable eThrowable) 
{
	// Save mnemonic 
	if (mnemonic != null)
		setMessage(mnemonic);

	// If debugMode and nested exception then get nested exception stack trace
	if ((debugMode) && (eThrowable != null))
	{
		nestedChainException  = eThrowable;
		if (nestedChainException instanceof WasdgException)
		{
			WasdgException e = (WasdgException) eThrowable;
			nestedStackTrace = e.getStackTraceString();
		}
		else
		{
			nestedStackTrace = generateStackTraceString(nestedChainException);
		}
	}

	// Update the stack trace string
	if ( null == nestedStackTrace )
		nestedStackTrace = generateStackTraceString(this);
	else
		nestedStackTrace = nestedStackTrace + generateStackTraceString(this);

	if (debugMode)
		Log.log( " -------- " 
		+ WasdgMessages.getComponentCode(mnemonic) + " " + WasdgMessages.getSeverityCode(mnemonic) + " " + mnemonic + ": " + WasdgMessages.getDescriptiveMessage(mnemonic)
		+ "\n" + nestedStackTrace
		+ "\n");
}

// Add a mnemonic to the list of messages for this exception
private void setMessage(String mnemonic) 
{
	arrayMessages.addElement(mnemonic);
}
}
