package com.tcnd.wasdg.common;

import java.io.*;
import org.w3c.dom.*;
import org.apache.soap.*;
import org.apache.soap.encoding.soapenc.*;
import org.apache.soap.rpc.*;
import org.apache.soap.util.*;
import org.apache.soap.util.xml.*;
import java.util.Hashtable;
import java.util.Vector;
import com.tcnd.wasdg.common.DataSet;

/*
inScopeEncStyle : is the encodingStyleURI as specified in the enclosing Call or Response object. 
javaType : is the run-time type of the object that is to be serialized. 
src : is a reference to the Java object to be serialized. 
context : is a String denoting the accessor name It must be non-null. 
sink : is the destination sink to which the SOAP XML instance will be written. 
nsStack : is a data structure that implements a stack of namespace declarations that are currently in scope. 
xjmr : is the XMLJavaMappingRegistry object. 
ctx : is used to pass in things like javax.servlet.http.HttpServletRequest and javax.servlet.http.HttpSession from the servlet context. 
*/


public class DataSetSerializer implements Serializer, Deserializer
{

public void marshall(String inScopeEncStyle, Class javaType, 
	Object src, Object context, Writer sink,
	NSStack nsStack, XMLJavaMappingRegistry xjmr, SOAPContext ctx) throws IllegalArgumentException, IOException
{
	nsStack.pushScope();

	SoapEncUtils.generateStructureHeader(inScopeEncStyle, javaType, context, sink, nsStack, xjmr);

	sink.write(StringUtils.lineSeparator);

	DataSet src2 = (DataSet)src;
	Parameter param;

	param = new Parameter("ht", Hashtable.class, new Hashtable(src2.ht), null);
	xjmr.marshall(inScopeEncStyle, Parameter.class, param, null, sink, nsStack, ctx);
	sink.write(StringUtils.lineSeparator);

	param = new Parameter("rows", Vector.class, new Vector(src2.rows), null);
	xjmr.marshall(inScopeEncStyle, Parameter.class, param, null, sink, nsStack, ctx);
	sink.write(StringUtils.lineSeparator);
	
	sink.write("</" + context + '>');
	nsStack.popScope();
}

public Bean unmarshall(String inScopeEncStyle, QName elementType, Node src, 
	XMLJavaMappingRegistry xjmr, SOAPContext ctx) throws IllegalArgumentException
{
	Element root = (Element)src;
	Element tempEl = DOMUtils.getFirstChildElement(root);
	DataSet target;

	try
	{
		target = (DataSet)DataSet.class.newInstance();
	}
	catch (Exception e)
	{
		throw new IllegalArgumentException("Problem instantiating bean: " + e.getMessage());
	}

	while (tempEl != null)
	{
		Bean paramBean = xjmr.unmarshall(inScopeEncStyle, RPCConstants.Q_ELEM_PARAMETER, tempEl, ctx);
		Parameter param = (Parameter)paramBean.value;
		String tagName = tempEl.getTagName();

		if (tagName.equals("ht"))
		{
			target.ht = (Hashtable)param.getValue();
		}

		if (tagName.equals("rows"))
		{
			target.rows = (Vector)param.getValue();
		}
		tempEl = DOMUtils.getNextSiblingElement(tempEl);
	}
	return new Bean(DataSet.class, target);
}
}
